@@ -47,13 +47,6 @@ impl Display {
         attrs.push(egl::HEIGHT as EGLint);
         attrs.push(height.get() as EGLint);
 
-        // Add information about render buffer.
-        attrs.push(egl::RENDER_BUFFER as EGLint);
-        let buffer =
-            if surface_attributes.single_buffer { egl::SINGLE_BUFFER } else { egl::BACK_BUFFER }
-                as EGLint;
-        attrs.push(buffer);
-
         // Push `egl::NONE` to terminate the list.
         attrs.push(egl::NONE as EGLint);
 
@@ -277,7 +270,7 @@ impl<T: SurfaceTypeTrait> Surface<T> {
         context.inner.bind_api();
 
         let res = unsafe {
-            if self.display.inner.client_extensions.contains("EGL_KHR_swap_buffers_with_damage") {
+            if self.display.inner.display_extensions.contains("EGL_KHR_swap_buffers_with_damage") {
                 self.display.inner.egl.SwapBuffersWithDamageKHR(
                     *self.display.inner.raw,
                     self.raw,
@@ -287,7 +280,7 @@ impl<T: SurfaceTypeTrait> Surface<T> {
             } else if self
                 .display
                 .inner
-                .client_extensions
+                .display_extensions
                 .contains("EGL_EXT_swap_buffers_with_damage")
             {
                 self.display.inner.egl.SwapBuffersWithDamageEXT(
@@ -340,7 +333,7 @@ impl<T: SurfaceTypeTrait> GlSurface<T> for Surface<T> {
     fn buffer_age(&self) -> u32 {
         self.display
             .inner
-            .client_extensions
+            .display_extensions
             .contains("EGL_EXT_buffer_age")
             .then(|| unsafe { self.raw_attribute(egl::BUFFER_AGE_EXT as EGLint) })
             .unwrap_or(0) as u32
@@ -481,7 +474,7 @@ impl NativeWindow {
                 }
 
                 let ptr = ffi_dispatch!(
-                    WAYLAND_EGL_HANDLE,
+                    wayland_egl_handle(),
                     wl_egl_window_create,
                     window_handle.surface.cast(),
                     _width.get() as _,
@@ -547,7 +540,7 @@ impl NativeWindow {
         if let Self::Wayland(wl_egl_surface) = self {
             unsafe {
                 ffi_dispatch!(
-                    WAYLAND_EGL_HANDLE,
+                    wayland_egl_handle(),
                     wl_egl_window_resize,
                     *wl_egl_surface as _,
                     _width.get() as _,
@@ -614,7 +607,7 @@ impl Drop for NativeWindow {
     fn drop(&mut self) {
         unsafe {
             if let Self::Wayland(wl_egl_window) = self {
-                ffi_dispatch!(WAYLAND_EGL_HANDLE, wl_egl_window_destroy, wl_egl_window.cast());
+                ffi_dispatch!(wayland_egl_handle(), wl_egl_window_destroy, wl_egl_window.cast());
             }
         }
     }
