@@ -41,20 +41,13 @@ pub trait NotCurrentGlContext: Sealed {
     /// The type of possibly current context.
     type PossiblyCurrentContext: PossiblyCurrentGlContext;
 
+    /// The surface supported by the context.
+    type Surface<T: SurfaceTypeTrait>: GlSurface<T>;
+
     /// Treat the not current context as possibly current. The operation is safe
     /// because the possibly current context is more restricted and not
     /// guaranteed to be current.
     fn treat_as_possibly_current(self) -> Self::PossiblyCurrentContext;
-}
-
-/// A trait that splits the methods accessing [`crate::surface::Surface`] on not
-/// current context.
-pub trait NotCurrentGlContextSurfaceAccessor<T: SurfaceTypeTrait>: Sealed {
-    /// The surface supported by the context.
-    type Surface: GlSurface<T>;
-    /// The possibly current context produced when making not current context
-    /// current.
-    type PossiblyCurrentContext: PossiblyCurrentGlContext;
 
     /// Make [`Self::Surface`] on the calling thread producing the
     /// [`Self::PossiblyCurrentContext`] indicating that the context could
@@ -62,19 +55,25 @@ pub trait NotCurrentGlContextSurfaceAccessor<T: SurfaceTypeTrait>: Sealed {
     ///
     /// # Platform specific
     ///
-    /// **macOS:** - **This will block if your main thread is blocked.**
-    fn make_current(self, surface: &Self::Surface) -> Result<Self::PossiblyCurrentContext>;
+    /// - **macOS: this will block if your main thread is blocked**;
+    /// - **Wayland:** this call may latch the underlying back buffer (will do
+    ///   with mesa drivers), meaning that all resize operations will apply
+    ///   after the next [`GlSurface::swap_buffers`].
+    fn make_current<T: SurfaceTypeTrait>(
+        self,
+        surface: &Self::Surface<T>,
+    ) -> Result<Self::PossiblyCurrentContext>;
 
     /// The same as [`Self::make_current`], but provides a way to set read and
     /// draw surfaces.
     ///
     /// # Api-specific:
     ///
-    /// **WGL/CGL:** - not supported.
-    fn make_current_draw_read(
+    /// - **WGL/CGL:** not supported.
+    fn make_current_draw_read<T: SurfaceTypeTrait>(
         self,
-        surface_draw: &Self::Surface,
-        surface_read: &Self::Surface,
+        surface_draw: &Self::Surface<T>,
+        surface_read: &Self::Surface<T>,
     ) -> Result<Self::PossiblyCurrentContext>;
 }
 
@@ -83,6 +82,9 @@ pub trait PossiblyCurrentGlContext: Sealed {
     /// The not current context type.
     type NotCurrentContext: NotCurrentGlContext;
 
+    /// The surface supported by the context.
+    type Surface<T: SurfaceTypeTrait>: GlSurface<T>;
+
     /// Returns `true` if this context is the current one in this thread.
     fn is_current(&self) -> bool;
 
@@ -92,32 +94,26 @@ pub trait PossiblyCurrentGlContext: Sealed {
     ///
     /// # Platform specific
     ///
-    /// **macOS:** - **This will block if your main thread is blocked.**
+    /// - **macOS: this will block if your main thread is blocked.**
     fn make_not_current(self) -> Result<Self::NotCurrentContext>;
-}
-
-/// A trait that splits the methods accessing [`crate::surface::Surface`].
-pub trait PossiblyCurrentContextGlSurfaceAccessor<T: SurfaceTypeTrait>: Sealed {
-    /// The surface supported by the context.
-    type Surface: GlSurface<T>;
 
     /// Make [`Self::Surface`] current on the calling thread.
     ///
     /// # Platform specific
     ///
-    /// **macOS:** - **This will block if your main thread is blocked.**
-    fn make_current(&self, surface: &Self::Surface) -> Result<()>;
+    /// - **macOS: this will block if your main thread is blocked.**
+    fn make_current<T: SurfaceTypeTrait>(&self, surface: &Self::Surface<T>) -> Result<()>;
 
     /// The same as [`Self::make_current`] but provides a way to set read and
     /// draw surfaces explicitly.
     ///
     /// # Api-specific:
     ///
-    /// **CGL/WGL:** - not supported.
-    fn make_current_draw_read(
+    /// - **CGL/WGL:** not supported.
+    fn make_current_draw_read<T: SurfaceTypeTrait>(
         &self,
-        surface_draw: &Self::Surface,
-        surface_read: &Self::Surface,
+        surface_draw: &Self::Surface<T>,
+        surface_read: &Self::Surface<T>,
     ) -> Result<()>;
 }
 
@@ -142,6 +138,7 @@ impl ContextAttributesBuilder {
     /// Sets the *debug* flag for the OpenGL context.
     ///
     /// Debug contexts are usually slower, but give better error reporting.
+    /// This option is ignored when using [`Robustness::NoError`].
     ///
     /// The default value for this flag is `false`.
     pub fn with_debug(mut self, debug: bool) -> Self {
@@ -156,7 +153,7 @@ impl ContextAttributesBuilder {
     ///
     /// # Platform-specific
     ///
-    /// On Wayland both contexts must use the same Wayland connection.
+    /// - **Wayland:** both contexts must use the same Wayland connection.
     ///
     /// [`Config`]: crate::config::Config
     pub fn with_sharing(mut self, context: &impl AsRawContext) -> Self {
@@ -190,7 +187,7 @@ impl ContextAttributesBuilder {
     ///
     /// # Api-specific
     ///
-    /// **macOS:** - not supported, the latest is picked automatically.
+    /// - **macOS:** not supported, the latest is picked automatically.
     pub fn with_profile(mut self, profile: GlProfile) -> Self {
         self.attributes.profile = Some(profile);
         self
@@ -210,8 +207,8 @@ impl ContextAttributesBuilder {
     ///
     /// # Api-specific
     ///
-    /// **WGL:** - you must pass `raw_window_handle` for if you plan to use it
-    /// with window.
+    /// - **WGL:** you **must** pass a `raw_window_handle` if you plan to use
+    ///   this context with that window.
     pub fn build(mut self, raw_window_handle: Option<RawWindowHandle>) -> ContextAttributes {
         self.attributes.raw_window_handle = raw_window_handle;
         self.attributes
@@ -239,10 +236,11 @@ pub struct ContextAttributes {
 /// Specifies the tolerance of the OpenGL context to faults. If you accept
 /// raw OpenGL commands and/or raw shader code from an untrusted source, you
 /// should definitely care about this.
-#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+#[derive(Debug, Copy, Clone, PartialEq, Eq, Default)]
 pub enum Robustness {
     /// Not everything is checked. Your application can crash if you do
     /// something wrong with your shaders.
+    #[default]
     NotRobust,
 
     /// The driver doesn't check anything. This option is very dangerous.
@@ -265,13 +263,6 @@ pub enum Robustness {
     RobustLoseContextOnReset,
 }
 
-impl Default for Robustness {
-    #[inline]
-    fn default() -> Self {
-        Robustness::NotRobust
-    }
-}
-
 /// Describes the requested OpenGL context profiles.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum GlProfile {
@@ -337,28 +328,22 @@ impl Version {
 }
 
 /// The behavior of the driver when you change the current context.
-#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+#[derive(Debug, Copy, Clone, PartialEq, Eq, Default)]
 pub enum ReleaseBehavior {
     /// Doesn't do anything. Most notably doesn't flush. Not supported by all
     /// drivers.
     ///
     /// # Api-specific
     ///
-    /// **macOS:** - not supported, [`Self::Flush`] is always used.
+    /// - **macOS:** not supported, [`Self::Flush`] is always used.
     None,
 
     /// Flushes the context that was previously current as if `glFlush` was
     /// called. This is the default behavior.
+    #[default]
     Flush,
 }
 
-impl Default for ReleaseBehavior {
-    #[inline]
-    fn default() -> Self {
-        ReleaseBehavior::Flush
-    }
-}
-
 /// A context that is known to be not current on the current thread.
 ///
 /// This type is a safe wrapper around the context to indicate that it could be
@@ -395,17 +380,16 @@ pub enum NotCurrentContext {
 
 impl NotCurrentGlContext for NotCurrentContext {
     type PossiblyCurrentContext = PossiblyCurrentContext;
+    type Surface<T: SurfaceTypeTrait> = Surface<T>;
 
     fn treat_as_possibly_current(self) -> Self::PossiblyCurrentContext {
         gl_api_dispatch!(self; Self(context) => context.treat_as_possibly_current(); as PossiblyCurrentContext)
     }
-}
 
-impl<T: SurfaceTypeTrait> NotCurrentGlContextSurfaceAccessor<T> for NotCurrentContext {
-    type PossiblyCurrentContext = PossiblyCurrentContext;
-    type Surface = Surface<T>;
-
-    fn make_current(self, surface: &Self::Surface) -> Result<Self::PossiblyCurrentContext> {
+    fn make_current<T: SurfaceTypeTrait>(
+        self,
+        surface: &Self::Surface<T>,
+    ) -> Result<Self::PossiblyCurrentContext> {
         match (self, surface) {
             #[cfg(egl_backend)]
             (Self::Egl(context), Surface::Egl(surface)) => {
@@ -427,10 +411,10 @@ impl<T: SurfaceTypeTrait> NotCurrentGlContextSurfaceAccessor<T> for NotCurrentCo
         }
     }
 
-    fn make_current_draw_read(
+    fn make_current_draw_read<T: SurfaceTypeTrait>(
         self,
-        surface_draw: &Self::Surface,
-        surface_read: &Self::Surface,
+        surface_draw: &Self::Surface<T>,
+        surface_read: &Self::Surface<T>,
     ) -> Result<Self::PossiblyCurrentContext> {
         match (self, surface_draw, surface_read) {
             #[cfg(egl_backend)]
@@ -521,6 +505,7 @@ pub enum PossiblyCurrentContext {
 
 impl PossiblyCurrentGlContext for PossiblyCurrentContext {
     type NotCurrentContext = NotCurrentContext;
+    type Surface<T: SurfaceTypeTrait> = Surface<T>;
 
     fn is_current(&self) -> bool {
         gl_api_dispatch!(self; Self(context) => context.is_current())
@@ -531,12 +516,8 @@ impl PossiblyCurrentGlContext for PossiblyCurrentContext {
             gl_api_dispatch!(self; Self(context) => context.make_not_current()?; as NotCurrentContext),
         )
     }
-}
 
-impl<T: SurfaceTypeTrait> PossiblyCurrentContextGlSurfaceAccessor<T> for PossiblyCurrentContext {
-    type Surface = Surface<T>;
-
-    fn make_current(&self, surface: &Self::Surface) -> Result<()> {
+    fn make_current<T: SurfaceTypeTrait>(&self, surface: &Self::Surface<T>) -> Result<()> {
         match (self, surface) {
             #[cfg(egl_backend)]
             (Self::Egl(context), Surface::Egl(surface)) => context.make_current(surface),
@@ -550,10 +531,10 @@ impl<T: SurfaceTypeTrait> PossiblyCurrentContextGlSurfaceAccessor<T> for Possibl
         }
     }
 
-    fn make_current_draw_read(
+    fn make_current_draw_read<T: SurfaceTypeTrait>(
         &self,
-        surface_draw: &Self::Surface,
-        surface_read: &Self::Surface,
+        surface_draw: &Self::Surface<T>,
+        surface_read: &Self::Surface<T>,
     ) -> Result<()> {
         match (self, surface_draw, surface_read) {
             #[cfg(egl_backend)]
